@@ -60,6 +60,9 @@ int verbose_on_data_chunk_recv_callback(nghttp2_session *session, uint8_t flags,
                                         int32_t stream_id, const uint8_t *data,
                                         size_t len, void *user_data);
 
+int verbose_error_callback(nghttp2_session *session, const char *msg,
+                           size_t len, void *user_data);
+
 // Returns difference between |a| and |b| in milliseconds, assuming
 // |a| is more recent than |b|.
 template <typename TimePoint>
