@@ -125,6 +125,9 @@ public:
 
   Worker *get_worker() const;
 
+  // Initializes forwarded_for_.
+  void init_forwarded_for(int family, const StringRef &ipaddr);
+
   using ReadBuf = DefaultMemchunkBuffer;
 
   ReadBuf *get_rb();
@@ -133,8 +136,6 @@ public:
   RateLimit *get_wlimit();
 
   void signal_write();
-  // Use this for HTTP/1 frontend since it produces better result.
-  void signal_write_no_wait();
   ev_io *get_wev();
 
   void setup_upstream_io_callback();
