@@ -41,6 +41,10 @@ namespace shrpx {
 
 struct MemcachedRequest;
 
+namespace tls {
+struct TLSSessionCache;
+} // namespace tls
+
 enum {
   TLS_CONN_NORMAL,
   TLS_CONN_WAIT_FOR_SESSION_CACHE,
@@ -55,6 +59,7 @@ struct TLSConnection {
   SSL *ssl;
   SSL_SESSION *cached_session;
   MemcachedRequest *cached_session_lookup_req;
+  tls::TLSSessionCache *client_session_cache;
   ev_tstamp last_write_idle;
   size_t warmup_writelen;
   // length passed to SSL_write and SSL_read last time.  This is
@@ -66,6 +71,9 @@ struct TLSConnection {
   bool reneg_started;
   // true if ssl is prepared to do handshake as server.
   bool server_handshake;
+  // true if ssl is initialized as server, and client requested
+  // signed_certificate_timestamp extension.
+  bool sct_requested;
 };
 
 struct TCPHint {
@@ -154,22 +162,16 @@ struct Connection {
   // used in this object at the moment.  The rest of the program may
   // use this value when it is useful.
   shrpx_proto proto;
-  // The point of time when last read is observed.  Note: sinde we use
+  // The point of time when last read is observed.  Note: since we use
   // |rt| as idle timer, the activity is not limited to read.
   ev_tstamp last_read;
   // Timeout for read timer |rt|.
   ev_tstamp read_timeout;
 };
 
-// Creates BIO_method shared by all SSL objects.  If nghttp2 is built
-// with OpenSSL < 1.1.0, this returns statically allocated object.
-// Otherwise, it returns new BIO_METHOD object every time.
+// Creates BIO_method shared by all SSL objects.
 BIO_METHOD *create_bio_method();
 
-// Deletes given |bio_method|.  If nghttp2 is built with OpenSSL <
-// 1.1.0, this function is noop.
-void delete_bio_method(BIO_METHOD *bio_method);
-
 } // namespace shrpx
 
 #endif // SHRPX_CONNECTION_H
